@@ -19,7 +19,7 @@ from future import standard_library
 
 from .common import *
 from .dcm import Component, Dcm
-from .sch import Schematic, Schematic_V6
+from .sch import sch_field_id_to_name, Schematic, Schematic_V6
 from .schlib import SchLib, SchLib_V6
 
 standard_library.install_aliases()
@@ -83,11 +83,18 @@ def group_wb(wb):
     """Group lines that have the same column values in a openpyxl workbook.
     Headers are expected on the first row and references are expected in the
     first column."""
+
     ws = wb.active
     values = tuple(ws.values)
+
+    try:
+        header = values[0]
+    except IndexError:
+        # No header, so don't even try to ungroup the workbook.
+        return wb
+
     unique_rows = []
     references = []
-    header = values[0]
     for row in values[1:]:
         column_values = row[1:]
         reference = row[0]
@@ -112,6 +119,33 @@ def group_wb(wb):
     return grouped_wb
 
 
+def ungroup_wb(wb):
+    """Ungroup lines that have collapsed references."""
+    # return wb
+
+    ws = wb.active
+    values = tuple(ws.values)
+
+    try:
+        header = values[0]
+    except IndexError:
+        # No header, so don't even try to ungroup the workbook.
+        return wb
+
+    ungrouped_wb = pyxl.Workbook()
+    ungrouped_ws = ungrouped_wb.active
+    ungrouped_ws.append(header)
+
+    for row in values[1:]:
+        column_values = row[1:]
+        reference = row[0]
+        for ref in explode(reference):
+            ungrouped_row = ((ref,) + column_values)
+            ungrouped_ws.append(ungrouped_row)
+
+    return ungrouped_wb
+
+
 class FieldExtractionError(Exception):
     pass
 
@@ -203,6 +237,8 @@ def extract_part_fields_from_wb(
 ):
     """Return a dictionary of part fields extracted from an XLSX workbook."""
 
+    wb = ungroup_wb(wb)
+
     part_fields = {}  # Start with an empty part dictionary.
 
     try:
@@ -725,6 +761,8 @@ def insert_part_fields_into_wb(part_fields_dict, wb, recurse=False):
     if wb is None:
         wb = pyxl.Workbook()  # No workbook given, so create one.
 
+    wb = ungroup_wb(wb)  # Ungroup any grouped references.
+
     ws = wb.active  # Get the active sheet from the workbook.
 
     def set_cell_format(cell):
@@ -1034,11 +1072,12 @@ def insert_part_fields_into_sch(
                             ),
                         )
 
-                # Remove any named fields with empty values.
+                # Keep only default fields and named fields with non-empty values.
+                default_field_ids = sch_field_id_to_name.keys()
                 component.fields = [
                     f
                     for f in component.fields
-                    if unquote(f.get("name", None)) in (None, "", "~")
+                    if f["id"] in default_field_ids
                     or unquote(f.get("ref", None)) not in (None, "")
                 ]
 
@@ -1116,10 +1155,10 @@ def insert_part_fields_into_sch_V6(
 
             # Get [V] (visible) or [I] (invisible) flag prepended to entire field name
             # or individual field value. If no flag, visibility is set to None.
-            field_vis, field_name = re.match("(\[([VI])\])?(.*)", field_name).group(
+            field_vis, field_name = re.match(r"(\[([VI])\])?(.*)", field_name).group(
                 2, 3
             )
-            value_vis, field_value = re.match("(\[([VI])\])?(.*)", field_value).group(
+            value_vis, field_value = re.match(r"(\[([VI])\])?(.*)", field_value).group(
                 2, 3
             )
             if value_vis == "V":
@@ -1313,10 +1352,10 @@ def insert_part_fields_into_lib_V6(
 
             # Get [V] (visible) or [I] (invisible) flag prepended to entire field name
             # or individual field value. If no flag, visibility is set to None.
-            field_vis, field_name = re.match("(\[([VI])\])?(.*)", field_name).group(
+            field_vis, field_name = re.match(r"(\[([VI])\])?(.*)", field_name).group(
                 2, 3
             )
-            value_vis, field_value = re.match("(\[([VI])\])?(.*)", field_value).group(
+            value_vis, field_value = re.match(r"(\[([VI])\])?(.*)", field_value).group(
                 2, 3
             )
             if value_vis == "V":
